angular.module('demoWidget6Module')

    /**
     * Controller for the Assign data dialog
     */
    .controller('demoWidget6AssignDataCtrl',['$scope','aggregationConstants','typeconstants','numberFormatConstants',
        function($scope, aggregationConstants, typeconstants, numberFormatConstants){

            /**
             * This function changes the selection state of a column
             * A column selection triggers the loading of the matching configuration view template.
             * Initially the threshold control is set to be hidden
             * @param column The column that is to be selected
             */
            $scope.columnSelectionHandler = function(column) {
                $scope.showThresholdView = false;
                $scope.selectedColumn = column;
            };

            /**
             * This function is called to enable the threshold properties view
             */
            $scope.thresholdSelectionHandler = function() {
                $scope.showThresholdView = true;
            };

            /**
             * This function is called when a column was dropped in the assign data view template
             * According to the column data type the column configuration will be predefined
             * @param column The column that was dropped
             */
            $scope.columnDropHandler = function(column) {
                column.newName = column.name;
                if(column.type === typeconstants.NUMBER){
                    if(!column.aggregation) {
                        column.aggregation = aggregationConstants.AVG;
                    }
                    if(!column.format) {
                        column.format = numberFormatConstants.NUMERIC_FORMAT_PATTERNS[3];
                    }
                    column.round = true;
                } else if(column.type === typeconstants.DATE){
                    column.format = "yyyy-MM-dd'T'HH:mm:ss";
                }
                $scope.columnSelectionHandler(column);
            };

            /**
             * This function is called when the user clicks on the trash icon of a column
             * If no column is selected the configuration view will be hidden.
             * @param column The column that should be deleted
             */
            $scope.columnDeleteHandler = function(column) {
                if ($scope.selectedColumn === column) {
                    delete $scope.selectedColumn;
                }
            };

            /**
             * Indicates if a column should be marked as selected
             * @param column The column to check
             * @returns {boolean} true if the threshold view is not visible and the given column matches the selected one, otherwise false
             */
            $scope.selectColumn = function(column) {
                //return $scope.selectedColumn === column;
                return $scope.showThresholdView ? false : $scope.selectedColumn === column;
            };

            /**
             * This function searches for assigned columns and selects the first one found
             */
            $scope.selectFirstColumnFound = function(){
                var foundColumn = undefined;
                //check numeric column
                if($scope.itemClone.config.assignedColumns.numDataColumn != undefined){
                    foundColumn = $scope.itemClone.config.assignedColumns.numDataColumn;
                } else
                //check text column
                if($scope.itemClone.config.assignedColumns.textDataColumn != undefined){
                    foundColumn = $scope.itemClone.config.assignedColumns.textDataColumn;
                } else
                //check date column
                if($scope.itemClone.config.assignedColumns.dateDataColumn != undefined){
                    foundColumn = $scope.itemClone.config.assignedColumns.dateDataColumn;
                } else
                //check other columns
                if($scope.itemClone.config.assignedColumns.furtherColumns != undefined && $scope.itemClone.config.assignedColumns.furtherColumns.length > 0){
                    foundColumn = $scope.itemClone.config.assignedColumns.furtherColumns[0];
                }

                if(foundColumn != undefined){
                    $scope.columnSelectionHandler(foundColumn);
                }

            };

            //There must be one selected column, initially select the first column found
            $scope.selectFirstColumnFound();

        }
    ]);