angular.module('demoWidget6Module')

	/**
	 * Controller for the custom widget
	 * The controller provides the widget variables and manages the interactions between data-model and view
	 */
    .controller('demoWidget6Ctrl',['$scope','formatDateService','formatNumberService','thresholdConstants','thresholdService','filterService',
        function($scope, formatDateService, formatNumberService, thresholdConstants, thresholdService, filterService){

            //Create scope variables to reference the services used for value transformations in the custom widget view
            $scope.formatDateService = formatDateService;
            $scope.formatNumberService = formatNumberService;

            //This array holds the current thresholds
            var thresholds = [];

            //The column-index-map is used to map the column names onto the original column indices of the feed result
            var columnIndexMap = {};
            //$scope.selectedRow holds the selected row
            $scope.selectedRow = undefined;

            /**
             * The init function creates all necessary scope variables at widget start up time
             */
            var initWidget = function() {
                //Create a scope object "config" if not present already
                //$scope.config holds the data of the widget settings like the assigned column(s)
                if(!$scope.config){
                    $scope.config = {};
                }
                //Define a property "assignedColumns" in the widget settings if not present already
                //to manage the information about the assigned column(s) in the controller
                if (!$scope.config.assignedColumns) {
                    $scope.config.assignedColumns = {};
                }

                //Define a property "advancedWidgetCfg" in the widget settings if not present already
                //to manage the information about the advanced column(s) configuration in the controller
                if (!$scope.config.advancedWidgetCfg) {
                    $scope.config.advancedWidgetCfg = {
                        thresholds: angular.copy(thresholdConstants.defaultThresholds)
                    };
                }
            };

            /**
             * This function searches for a configured column with a given data source column name
             * @param columnName The name of the data source column
             * @retrun The configured column, or undefined if not found
             */
            var getConfiguredColumnByName = function(columnName) {
                //check numeric column
                if($scope.config.assignedColumns.numDataColumn != undefined && $scope.config.assignedColumns.numDataColumn.newName == columnName){
                    return $scope.config.assignedColumns.numDataColumn;
                }
                //check text column
                if($scope.config.assignedColumns.textDataColumn != undefined && $scope.config.assignedColumns.textDataColumn.newName == columnName){
                    return $scope.config.assignedColumns.textDataColumn;
                } else
                //check date column
                if($scope.config.assignedColumns.dateDataColumn != undefined && $scope.config.assignedColumns.dateDataColumn.newName == columnName){
                    return $scope.config.assignedColumns.dateDataColumn;
                } else
                //check further columns
                if($scope.config.assignedColumns.furtherColumns != undefined){
                    for(var i = 0; i < $scope.config.assignedColumns.furtherColumns.length; i++){
                        var column = $scope.config.assignedColumns.furtherColumns[i];
                        if(column.newName == columnName){
                            return $scope.config.assignedColumns.furtherColumns[i];
                        }
                    }
                }
                return undefined;
            };

            /**
             * This function adds all configured columns to a scope variable configuredColumns
             * that can be used by the custom widget view to access the column configuration
             * This function also creates a column-index-map to determine the index of a column by its name.
             * @param data The data for the assigned and server-requested data source columns
             */
            var createConfiguredColumns = function(data) {
                for (var i = 0; i < data.columns.length; i++) {
                    var configuredColumn = getConfiguredColumnByName(data.columns[i].name);
                    var columnName = data.columns[i].name;
                    if(configuredColumn != undefined){
                        $scope.configuredColumns[columnName] = configuredColumn;
                        columnIndexMap[columnName] = i;
                        $scope.columns.push(data.columns[i]);
                    }
                }
            };

            /**
             * This function transforms the column und row data coming from the server
             * and creates scope variables that can be easily used in the view
             * @param data The data for the assigned and server-requested data source columns
             */
            var createScopeVariables = function(data) {
                $scope.columns = [];
                $scope.rows = data.rows;
                $scope.rowCount = data.rows.length;
                $scope.configuredColumns = [];
            };

            var getRGBAColor = function (color, alpha){
                var c = color.substring(1);      // strip #
                var rgb = parseInt(c, 16);   // convert rrggbb to decimal
                var r = (rgb >> 16) & 0xff;  // extract red
                var g = (rgb >>  8) & 0xff;  // extract green
                var b = (rgb >>  0) & 0xff;  // extract blue
                return 'rgba('+r+','+g+','+b+','+alpha+')';
            };

            /**
             * This function returns the threshold color for a matching value
             * @param thresholdIndex The row index pointing to the corresponding threshold in the threshold array
             * @param row The row data object
             * @return the threshold color or undefined if not found
             */
            $scope.getRowColor = function(thresholdIndex, row){
                if(thresholds[thresholdIndex]) {
                    var thresholdColumn = $scope.config.assignedColumns.numDataColumn;
                    if (thresholdColumn != undefined) {
                        var thresholdColumnIdx = -1;
                        for (var i = 0; i < $scope.columns.length; i++) {
                            if ($scope.columns[i].name == thresholdColumn.newName) {
                                thresholdColumnIdx = $scope.columns[i].idx;
                                break;
                            }
                        }
                        var thresholdColumnCellValue = row.values[thresholdColumnIdx];
                        for (var i = 0; i < thresholds[thresholdIndex].length; i++) {
                            if (thresholds[thresholdIndex][i].containsValue(thresholdColumnCellValue)) {
                                //return getRGBAColor(thresholds[thresholdIndex][i].colour, 0.5);
                                return thresholds[thresholdIndex][i].colour;
                            }
                        }
                    }
                }
                return undefined; //'transparent';
            };

            /**
             * This function collects the threshold colors for all rows
             * @param data The data for the assigned and server-requested data source columns (feed result)
             */
            var createThresholds = function(data) {
                thresholds = [];

                for(var i = 0; i < data.rows.length; i++){
                    var row = data.rows[i];
                    if($scope.config.advancedWidgetCfg.thresholds && data) {
                        var thresholdAreas = thresholdService.getThresholdAreas($scope.config.advancedWidgetCfg.thresholds, data, row);
                        thresholds.push(thresholdAreas);
                    }
                }
            };

            /**
             * This function checks if the current feed result contains the selected value
             * If no matching selection value was found the widget selection will be removed
             * @param data The data for the assigned and server-requested data source columns (feed result)
             */
            var checkSelection = function(data){

                var found = false;
                for(var i = 0; i < data.rows.length; i++){
                    var row = data.rows[i];
                    if(!row.values || row.values.length < 1){
                        continue;
                    }
                    var val = row.values[0];
                    if(val === $scope.selectedRow) {
                        found = true;
                        break;
                    }
                }
                if(!found) {
                    filterService.onSelectionChange($scope.item.identifier, [], []);
                    $scope.selectedRow = undefined;
                }
            };

            /**
             * This helper function returns the value for a certain column in a given row
             * @param row The row object
             * @param colName The name of the column
             * @returns The value of the row column or undefined if the row has no data
             */
            var getCellValue = function(row, colName){

                if(!row || !row.values){
                    return undefined;
                }
                return row.values[columnIndexMap[colName]];
            };

            /**
             * This helper function collects column names and column values for a given row
             * and fills the given arrays columnNames and columnValues with the collected data
             * @param row The given row
             * @param ioColumnNames The given array for collected column names
             * @param ioColumnValues The given array for collected column values
             */
            var fillRowDataArrays = function(row, ioColumnNames, ioColumnValues){

                if(row === undefined){
                    return;
                }
                for(var i=0; i < $scope.columns.length; i++){
                    var configuredColumnName = $scope.configuredColumns[$scope.columns[i].name].newName;
                    ioColumnNames.push(configuredColumnName);
                    ioColumnValues.push(getCellValue(row, configuredColumnName));
                }
            };

            /**
             * This helper function informs the framework about selection changes
             * @param row The selected row
             */
            var handleRowSelection = function(row) {

                $scope.selectedRow = row;

                var columnNames = [];
                var columnValues = [];
                fillRowDataArrays(row, columnNames, columnValues);
                filterService.onSelectionChange($scope.item.identifier, columnNames, columnValues, $scope.cardIdentifier);
            };

            /**
             * This function is called when the user selects a row in the custom widget
             * @param row The selected row
             */
            $scope.onRowClick = function(row) {

                handleRowSelection(row);
            }

            /**
             * This event listener is called from the custom widget framework when the user clicks "Clear selection" in the widget menu
             * The event listener function clears the selection in the custom widget
             */
            $scope.$on("onDeleteSelection", function(){

                handleRowSelection();
            });

            /**
             * This function is called from the widget framework to set a selection in the custom widget
             * It searches all custom widget rows to find one matching the given selection parameters
             * @param cols A list of all the columns in selection
             * @param values A list of the values selected corresponding to cols
             */
            $scope.setSelection = function(cols, values) {

                if(!cols || !values || cols.length !== values.length) {
                    return;
                }

                var matchingRow = undefined;

                //loop through all the rows
                if(cols.length > 0) {
                    if($scope.rows) {
                        for (var i = 0; i < $scope.rows.length; i++) {
                            var rowUnderInspection = $scope.rows[i];

                            var matchingRowFound = true;
                            //loop through all columns that are in selection
                            for (var j = 0; j < cols.length; j++) {
                                var columnName = cols[j],
                                    columnIndex = columnIndexMap[columnName];

                                //if any mismatch is found, consider it as false and break
                                //check for valid column first
                                if (columnIndex === undefined){
                                    matchingRowFound = false;
                                    break;
                                }

                                var rowValue = rowUnderInspection.values[columnIndex];
                                var selectedValue = values[j];

                                //parse numeric columns values
                                var configuredColumnType = $scope.configuredColumns[columnName].type;
                                if(configuredColumnType === 'NUMERIC'){
                                    rowValue = parseFloat(rowUnderInspection.values[columnIndex]);
                                    selectedValue = parseFloat(values[j]);
                                }

                                if(rowValue !== selectedValue) {
                                    matchingRowFound = false;
                                    break;
                                }
                            }

                            if (matchingRowFound) {
                                matchingRow = rowUnderInspection;
                                break;
                            }
                        }
                    }
                }

                //select the matching row
                handleRowSelection(matchingRow);
            };

            /**
             * This function is called from the custom widget framework when the requested data from the server returns
             * @param data The data for the assigned and server-requested data source columns (feed result)
             */
            $scope.setData = function(feedResult){
                createScopeVariables(feedResult);
                createConfiguredColumns(feedResult);
                createThresholds(feedResult);
                checkSelection(feedResult);
            };

            /**
             * This function loads the custom widget settings at widget start up time and stores it in the controller
             * @param config The persisted custom widget settings from the dashboard definition
             */
            $scope.setConfig = function(config){
                $scope.config = config;
            };

            initWidget();
        }
    ]);